import { Article, escape, Issue, literal, Op } from '@magloft/db-client'
import { Controller, Get, PathParams, QueryParams } from '@tsed/common'
import { Returns } from '@tsed/schema'
import { ArticleSearchEntity, ArticleWithSearch } from '../../entities'
import { searchMatch } from '../../lib/text'

@Controller('/articles')
export class ArticleController {

  @Get('/:publicationId/search')
  @Returns(200, Array).Of(ArticleSearchEntity)
  async search(@PathParams('publicationId') publicationId: number, @QueryParams('searchText') searchText: string): Promise<ArticleSearchEntity[]> {
    const searchTerms = searchText.split(' ')
    const { name } = Article
    const articles = await Article.findAll({
      attributes: [
        'id',
        'title',
        'issueId',
        'text',
        'cover',
        [literal(`MATCH (${name}.title, ${name}.info, ${name}.text) AGAINST(${escape(searchText)} IN NATURAL LANGUAGE MODE)`), 'score']
      ],
      where: { publicationId, date: { [Op.ne]: null }, standalone: true },
      order: [[literal('score'), 'DESC']],
      limit: 20,
      include: [{ model: Issue, as: 'Issue', required: false, attributes: ['title', 'name', 'cover'] }]
    }) as ArticleWithSearch[]
    return articles
      .filter((article) => article.getDataValue('score') > 0)
      .map((article) => ArticleSearchEntity.from(article, { match: searchMatch(article.text, searchTerms) }))
  }
}
