import { Pwa, Reader } from '@magloft/db-client'
import { Controller, Get, PathParams, Req, Res } from '@tsed/common'
import { NotFound } from '@tsed/exceptions'
import { Returns } from '@tsed/schema'
import { AuthRequest } from '../../middlewares/AuthMiddleware'
import { PaymentService } from '../../services/PaymentService'

@Controller('/readers')
export class ReaderController {
  constructor(private payment: PaymentService) { }

  @Get('/:pwaId/:customerId/billing-redirect')
  @Returns(302)
  async getBillingRedirect(@PathParams('pwaId') pwaId: number, @PathParams('customerId') customerId: string, @Req() { headers }: AuthRequest, @Res() res: Res): Promise<void> {
    const pwa = await Pwa.findById(pwaId)
    if (!pwa) { throw new NotFound('PWA not found') }
    const provider = this.payment.getProvider(pwa, 'stripe')
    const returnUrl = headers['referer']
    const reader = await Reader.findOne({ where: { stripeId: customerId, publicationId: pwa.publicationId } })
    if (!reader) { throw new NotFound('Reader not found') }
    const session = await provider.createBillingPortalSession({ reader, returnUrl })
    res.redirect(302, session.url)
  }
}
