import { Pwa } from '@magloft/db-client'
import { Controller, Post, Req } from '@tsed/common'
import { NotFound } from '@tsed/exceptions'
import { Returns } from '@tsed/schema'
import Stripe from 'stripe'
import { ReqWithStripeEvent, VerifyStripe } from '../../middlewares/VerifyStripeMiddleware'
import { PaymentService } from '../../services/PaymentService'

@Controller('/webhooks')
export class WebhookController {
  constructor(private payment: PaymentService) { }

  @Post('/stripe')
  @VerifyStripe('connect')
  @Returns(200)
  postStripe(@Req() { event }: ReqWithStripeEvent): Promise<void> {
    switch (event.type) {
      case 'checkout.session.completed': {
        return this.checkoutSessionCompleted(event)
      }
    }
  }

  private async checkoutSessionCompleted(event: Stripe.Event) {
    const stripeSession = event.data.object as Stripe.Checkout.Session
    const pwaId = parseInt(stripeSession.metadata.pwaId, 10)
    const pwa = await Pwa.findById(pwaId)
    if (!pwa) { throw new NotFound('PWA not found') }
    const provider = this.payment.getProvider(pwa, 'stripe')
    const session = await provider.retrieveCheckoutSession(stripeSession)
    await this.payment.completeCheckoutSession(session)
  }
}
