import { CustomFont } from '@magloft/db-client'
import { Allow, CollectionOf, Description, Example, Property, Required, Schema } from '@tsed/schema'
import { Entity } from '../lib/Entity'

@Schema({ title: 'Custom Font' })

export class CustomFontEntity extends Entity {
  @Property()
  @Description('Unique ID')
  @Example(1)
  id: number

  @Required()
  @Description('Identifier')
  @Example('lobster')
  identifier: string

  @Required()
  @Description('Title')
  @Example('Lobster')
  get title() { return (this.model as CustomFont).title }

  @Required()
  @Description('Key')
  @Example('reader/fonts/lobster.css')
  get key() { return (this.model as CustomFont).key }

  @Required()
  @Description('Url')
  @Example('https://cdn.magloft.com/reader/fonts/lobster.css')
  get url() { return (this.model as CustomFont).url }

  @Property()
  @Description('public')
  @Example(false)
  public: boolean

  @CollectionOf(String)
  @Allow('normal-italic', 'bold-italic', 'normal-normal', 'bold-normal')
  @Description('Variants')
  @Example(['normal-italic', 'bold-italic', 'normal-normal', 'bold-normal'])
  variants: string[]

  @Property()
  @Description('Created at')
  createdAt: Date

  @Property()
  @Description('Updated at')
  updatedAt: Date
}
