import { Description, Example, Property, Schema } from '@tsed/schema'
import Stripe from 'stripe'
import { Entity } from '../lib/Entity'

@Schema({ title: 'Flexpdf Invoice' })
export class FlexpdfInvoiceEntity extends Entity {
  sourceInvoice: Stripe.Invoice
  paidAt: Date | null
  cancelledAt: Date | null
  createdAt: Date | null

  @Property()
  @Description('Unique ID')
  @Example(1)
  id: number

  @Property()
  @Description('Invoice Source')
  @Example('stripe')
  source: string

  @Property()
  @Description('Invoice Status')
  @Example('paid')
  get status() { return this.sourceInvoice.status }

  @Property()
  @Description('Invoice Amount in USD cents')
  @Example(1000)
  amount: number

  @Property()
  @Description('Invoice Credits')
  @Example(100)
  credits: number

  @Property()
  @Description('Invoice ID')
  @Example('CF712-174234')
  get number() { return this.sourceInvoice.number }

  @Property()
  @Description('Invoice Url')
  @Example('https://pay.stripe.com/invoice/invst_abcdefghijklmnop0123456789')
  get invoiceUrl(): string { return this.sourceInvoice.hosted_invoice_url ?? '' }

  @Property()
  @Description('Receipt Url')
  @Example('https://pay.stripe.com/invoice/invst_abcdefghijklmnop0123456789/pdf')
  get receiptUrl(): string { return this.sourceInvoice.hosted_invoice_url ? `${this.sourceInvoice.hosted_invoice_url}/pdf` : '' }

  @Property()
  @Description('Invoice Paid Date')
  get paidAtTime() { return this.paidAt ? Math.floor(this.paidAt.getTime() / 1000) : null }

  @Property()
  @Description('Invoice Cancelled Date')
  get cancelledAtTime() { return this.cancelledAt ? Math.floor(this.cancelledAt.getTime() / 1000) : null }

  @Property()
  @Description('Invoice Created Date')
  get createdAtTime() { return this.createdAt ? Math.floor(this.createdAt.getTime() / 1000) : null }
}
