import { Description, Enum, Example, Property, Schema } from '@tsed/schema'
import { Entity } from '../lib/Entity'

export enum IssueSource { PDF = 'pdf', TYPELOFT = 'typeloft', TYPELOFT2 = 'typeloft2', EPUB = 'epub', HPUB = 'hpub', FOLIO = 'folio', ARTICLE = 'article' }
export enum IssueUnlockType { FREE = 'free', PAID = 'paid', HIDDEN = 'hidden', SUBSCRIBE = 'subscribe', OPTIN = 'optin' }
export enum IssueGenerator { IMPORT = 'import', FLEXPDF = 'flexpdf' }

@Schema({ title: 'Issue' })
export class IssueEntity extends Entity {
  private createdAt: Date | null
  private date: Date | null
  private cover: string | null

  @Property()
  @Description('Unique ID')
  @Example(1)
  id: number

  @Property()
  @Description('Publication ID')
  @Example(1)
  publicationId: number

  @Property()
  @Description('User ID')
  @Example(1)
  userId: number

  @Property()
  @Description('Title')
  @Example('My First Issue')
  title: string

  @Property()
  @Description('Info')
  @Example('Issue Description')
  info: string

  @Property()
  @Description('Slug')
  @Example('my-first-issue')
  slug: string

  @Property()
  @Enum(IssueSource)
  source: IssueSource

  @Property()
  @Enum(IssueUnlockType)
  unlockType: IssueUnlockType

  @Property()
  @Enum(IssueGenerator)
  generator: IssueGenerator

  @Property()
  @Description('Cover')
  @Example('https://cdn.magloft.com/defaults/images/cover.jpg')
  get coverUrl() { return `https://cdn.magloft.com/${this.cover || 'defaults/images/cover.jpg'}` }

  @Property()
  @Description('Created Date')
  get createdAtTime() { return this.createdAt ? Math.floor(this.createdAt.getTime() / 1000) : null }

  @Property()
  @Description('Publish Date')
  get dateTime() { return this.date ? Math.floor(this.date.getTime() / 1000) : null }
}
