import { Publication } from '@magloft/db-client'
import { Description, Example, Property, Schema } from '@tsed/schema'
import { Entity } from '../lib/Entity'

@Schema({ title: 'Publication' })
export class PublicationEntity extends Entity {
  private createdAt: Date | null
  private properties: Publication['properties']

  @Property()
  @Description('Unique ID')
  @Example(1)
  id: number

  @Property()
  @Description('App ID')
  @Example('com.magloft.magazine')
  appId: string

  @Property()
  @Description('Title')
  @Example('My First Publication')
  title: string

  @Property()
  @Description('Slug')
  @Example('magloft')
  slug: string

  @Property()
  @Description('Icon')
  @Example('https://cdn.magloft.com/images/defaults/app-icon.png')
  get iconUrl() { return `https://cdn.magloft.com/${this.properties.get('appIcon') ?? 'images/defaults/app-icon.png'}` }

  @Property()
  @Description('Created Date')
  get createdAtTime() { return this.createdAt ? Math.floor(this.createdAt.getTime() / 1000) : null }
}
