import { Pwa } from '@magloft/db-client'
import { CollectionOf, Description, Example, Property, Schema } from '@tsed/schema'

@Schema({ title: 'Push Website JSON' })
export class PushWebsiteJsonEntity {
  static fromPwa(pwa: Pwa, env = 'development', authenticationToken: string) {
    const instance = new PushWebsiteJsonEntity()
    instance.websiteName = pwa.design.strings.title
    instance.websitePushID = `web.${pwa.domain.split('.').reverse().join('.')}`
    instance.allowedDomains = [`https://${pwa.domain}`]
    instance.urlFormatString = `https://${pwa.domain}/%@/%@`
    instance.authenticationToken = authenticationToken
    instance.webServiceURL = `https://${env === 'development' ? 'api.magloft.dev' : 'api.magloft.com'}/pwa/push`
    return instance
  }

  @Property()
  @Description('Website Name')
  @Example('MagLoft PWA')
  websiteName: string

  @Property()
  @Description('Website Push ID')
  @Example('web.com.magloft.pwa')
  websitePushID: string

  @CollectionOf(String)
  @Description('Allowed Domains')
  @Example(['https://pwa.magloft.com'])
  allowedDomains: string[]

  @Property()
  @Description('URL Format String')
  @Example('https://pwa.magloft.com/%@/%@')
  urlFormatString: string

  @Property()
  @Description('Authentication Token')
  @Example('0000000100000000')
  authenticationToken: string

  @Property()
  @Description('Web Service URL')
  @Example('https://api.magloft.com/pwa/push/1')
  webServiceURL: string

  toJson() {
    return JSON.stringify(this, null, 2)
  }

  toBuffer() {
    return Buffer.from(this.toJson(), 'utf8')
  }
}
