import { BaseModel } from '@magloft/db-client/lib/models/base/Model.base'
import { CollectionOf, Description, Example, Name, Property, Schema } from '@tsed/schema'
import { Entity } from '../lib/Entity'

@Schema({ title: 'Reader' })
export class ReaderEntity extends Entity {
  constructor(model?: BaseModel, attributes?: object) {
    super(model, attributes)
    if (!this.customData) { this.customData = {} }
    if (!this.classificationIds) { this.classificationIds = [] }
  }

  @Property()
  @Description('Unique ID')
  @Example(1)
  id: number

  @Property()
  @Description('External ID')
  @Example('X91824')
  externalId: string

  @Property()
  @Description('Publication ID')
  @Example(1)
  publicationId: number

  @Property()
  @Description('Email Address')
  @Example('reader@magloft.com')
  email: string

  @Property()
  @Description('Password')
  @Example('FxhrRn')
  password?: string

  @Property()
  @Name('firstname')
  @Description('First name')
  @Example('John')
  name: string

  @Property()
  @Name('lastname')
  @Description('Last name')
  @Example('Snow')
  lastname: string

  @Property()
  @Description('Source')
  @Example('custom')
  source: string

  @Property()
  @Description('Custom data')
  @Example({ 'key': 'value' })
  customData?: { [key: string]: string }

  @CollectionOf(String)
  @Example(['C1', 'C2', 'C3'])
  classificationIds?: string[]

  @Property()
  @Description('Created at')
  createdAt: Date

  @Property()
  @Description('Updated at')
  updatedAt: Date
}
