import { User } from '@magloft/db-client'
import { Description, Example, Name, Property, Schema } from '@tsed/schema'
import * as crypto from 'crypto'
import { Entity } from '../lib/Entity'

@Schema({ title: 'User' })
export class UserEntity extends Entity {
  private properties: User['properties']

  @Property()
  @Description('Unique ID')
  @Example(1)
  id: number

  @Property()
  @Description('Email Address')
  @Example('user@magloft.com')
  email: string

  @Property()
  @Description('First Name')
  @Example('John')
  firstname: string

  @Property()
  @Description('Last Name')
  @Example('Snow')
  lastname: string

  @Property()
  @Description('Full Name')
  @Example('John Snow')
  get fullname(): string {
    return (this.firstname && this.lastname)
      ? `${this.firstname} ${this.lastname}`
      : this.firstname || this.lastname || 'Anonymous'
  }

  @Property()
  @Name('credits')
  @Description('Credits')
  @Example(500)
  flexpdfCredits: number

  @Property()
  @Description('Access Token')
  @Example('1$1234123412341234')
  get accessToken() {
    const hash = crypto.createHash('sha1')
    hash.update(`m&kv)Dn=02NCn2-${this.id}`)
    const digest = hash.digest('hex')
    return `${this.id}$${digest.substr(7, 16)}`
  }

  @Property()
  @Description('MMS ID')
  get mmsId() { return this.properties.get('mmsId') }

  @Property()
  @Description('MMS Token')
  get mmsToken() { return this.properties.get('mmsToken') }

  @Property()
  @Description('Country Code')
  get countryCode() { return this.properties.get('countryCode') ?? '' }

  @Property()
  @Description('VAT Number')
  get vatNumber() { return this.properties.get('vatNumber') ?? '' }
}
