import * as db from '@magloft/db-client'
import { ResolverService } from '@tsed/graphql'
import { Args, Authorized, Ctx, Query } from 'type-graphql'
import { Article, ArticleList } from '../../articles/Article'
import { AdminArticleDataSource } from './AdminArticleDataSource'
import { GetByIdArgs, GetListArgs } from './AdminArticleInput'

interface DataSources {
  adminArticleDataSource: AdminArticleDataSource
}

@ResolverService(Article)
export class AdminArticleResolver {
  @Authorized('user')
  @Query(() => Article)
  public async adminArticleGetById(@Ctx('dataSources') { adminArticleDataSource }: DataSources, @Ctx('user') user: db.User, @Args() args: GetByIdArgs): Promise<Article> {
    return adminArticleDataSource.getById(args, user)
  }

  @Authorized('user')
  @Query(() => ArticleList)
  public async adminArticleGetList(@Ctx('dataSources') { adminArticleDataSource }: DataSources, @Ctx('user') user: db.User, @Args() args: GetListArgs): Promise<ArticleList> {
    return adminArticleDataSource.getList(args, user)
  }
}
