import * as db from '@magloft/db-client'
import { fromString } from 'html-to-text'
import { Field, Int, ObjectType } from 'type-graphql'
import { BaseSchema } from '../../lib/graphql/BaseSchema'

@ObjectType()
export class Article extends BaseSchema<db.Article> {
  @Field(type => Int) public id: number
  @Field() public slug: string
  @Field() public title: string
  @Field() public standalone: boolean
  @Field({ nullable: true }) public html?: string
  @Field({ nullable: true }) public info?: string
  @Field() public unlockType: string
  @Field() public source: string
  @Field() public text: string
  @Field(type => Int) public position: number
  @Field({ nullable: true }) public cover?: string
  @Field(type => Int) public publicationId: number
  @Field({ nullable: true }) public date?: Date
  @Field(type => Int, { nullable: true }) public issueId?: number
  @Field(type => [Int]) public categoryIds: number[]
  @Field(type => Int, { nullable: true }) pwaOptinId?: number

  constructor(model: db.Article) {
    super(model)
    if (!this.categoryIds) { this.categoryIds = [] }
    if (!this.text) { this.text = fromString(this.html, { wordwrap: false }) }
    if (!this.slug) { this.slug = `article-${this.id}` }
  }
}

@ObjectType()
export class ArticleList {
  @Field(type => Int) public offset: number
  @Field(type => Int) public limit: number
  @Field(type => [Article]) public records: Article[]
}
