import { ResolverService } from '@tsed/graphql'
import { Args, Ctx, Query } from 'type-graphql'
import { Article, ArticleList } from './Article'
import { ArticleDataSource } from './ArticleDataSource'
import { GetByIdArgs, GetBySlugArgs, GetListArgs } from './ArticleInput'

interface DataSources {
  articleDataSource: ArticleDataSource
}

@ResolverService(Article)
export class ArticleResolver {
  @Query(() => Article)
  public async articleGetById(@Ctx('dataSources') { articleDataSource }: DataSources, @Args() args: GetByIdArgs): Promise<Article> {
    return articleDataSource.getById(args)
  }

  @Query(() => Article)
  public async articleGetBySlug(@Ctx('dataSources') { articleDataSource }: DataSources, @Args() args: GetBySlugArgs): Promise<Article> {
    return articleDataSource.getBySlug(args)
  }

  @Query(() => ArticleList)
  public async articleGetList(@Ctx('dataSources') { articleDataSource }: DataSources, @Args() args: GetListArgs): Promise<ArticleList> {
    return articleDataSource.getList(args)
  }
}
