import * as db from '@magloft/db-client'
import { DataSourceService } from '@tsed/graphql'
import { DataSource } from 'apollo-datasource'
import { Category } from './Category'
import { GetByIdArgs, GetBySlugArgs } from './CategoryInput'

export type CategoryWithArticles = db.Category & { Articles: db.Article[] }

@DataSourceService()
export class CategoryDataSource extends DataSource {
  public async getById({ id }: GetByIdArgs): Promise<Category> {
    // Validate Category
    const category = await db.Category.findById(id)
    if (!category) { throw new Error('Category not found') }

    // Return Category
    const entity = new Category(category)
    entity.image = await this.getCategoryImage(category.id)
    return entity
  }

  public async getBySlug({ slug }: GetBySlugArgs): Promise<Category> {
    // Validate Category
    const category = await db.Category.findOne({ where: { slug } })
    if (!category) { throw new Error('Category not found') }

    // Return Category
    const entity = new Category(category)
    entity.image = await this.getCategoryImage(category.id)
    return entity
  }

  private async getCategoryImage(categoryId: number) {
    const { Op, fn, col } = db
    const where = {
      [Op.and]: [
        db.where(fn('JSON_CONTAINS', col('Article.category_ids'), JSON.stringify([categoryId])), '=', '1'),
        { date: { [Op.ne]: null }, html: { [Op.ne]: null }, standalone: true }
      ]
    }
    const article = await db.Article.findOne({ where, attributes: ['id', 'cover'], order: [['date', 'desc']] })
    return article ? article.cover : null
  }
}
