import * as db from '@magloft/db-client'
import { Field, Float, Int, ObjectType } from 'type-graphql'
import { IssueWithArticles } from '../../api/pwa/SitemapController'
import { BaseSchema } from '../../lib/graphql/BaseSchema'
import { Article } from '../articles/Article'

@ObjectType()
export class Issue extends BaseSchema<db.Issue> {
  @Field(type => Int) public id: number
  @Field() public slug: string
  @Field() public title: string
  @Field({ nullable: true }) public get hpubUrl(): string { return this.model.hpub ? `https://cdn.magloft.com/${this.model.hpub}` : null }
  @Field() public get coverUrl(): string { return `https://cdn.magloft.com/${this.model.cover}` }
  @Field() public unlockType: string
  @Field(type => Int) public publicationId: number
  @Field() public createdAt: Date
  @Field() public updatedAt: Date
  @Field() public source: String
  @Field(type => [String], { defaultValue: [] }) public screenshots: string[]
  @Field({ nullable: true }) public productId?: string
  @Field({ nullable: true }) public name?: string
  @Field({ nullable: true }) public info?: string
  @Field({ nullable: true }) public date?: Date
  @Field(type => Float) public get price(): number { return this.model.priceTier ? this.model.priceTier - 0.01 : 0 }
  @Field(type => [Int]) public categoryIds: number[]
  @Field(type => [Article]) public articles: Article[]

  constructor(model: db.Issue) {
    super(model)
    if (!this.categoryIds) { this.categoryIds = [] }
    if (!this.screenshots) { this.screenshots = [] }
    this.articles = ((model as IssueWithArticles).Articles ?? []).map((article) => new Article(article))
  }
}

@ObjectType()
export class IssueList {
  @Field(type => Int) public offset: number
  @Field(type => Int) public limit: number
  @Field(type => [Issue]) public records: Issue[]
}
