import { ResolverService } from '@tsed/graphql'
import { Args, Ctx, Query } from 'type-graphql'
import { Issue, IssueList } from './Issue'
import { IssueDataSource } from './IssueDataSource'
import { GetByIdArgs, GetBySlugArgs, GetListArgs } from './IssueInput'

interface DataSources {
  issueDataSource: IssueDataSource
}

@ResolverService(Issue)
export class IssueResolver {
  @Query(() => Issue)
  public async issueGetById(@Ctx('dataSources') { issueDataSource }: DataSources, @Args() args: GetByIdArgs): Promise<Issue> {
    return issueDataSource.getById(args)
  }

  @Query(() => Issue)
  public async issueGetBySlug(@Ctx('dataSources') { issueDataSource }: DataSources, @Args() args: GetBySlugArgs): Promise<Issue> {
    return issueDataSource.getBySlug(args)
  }

  @Query(() => IssueList)
  public async issueGetList(@Ctx('dataSources') { issueDataSource }: DataSources, @Args() args: GetListArgs): Promise<IssueList> {
    return issueDataSource.getList(args)
  }
}
