import { GraphQLJSONObject } from 'graphql-type-json'
import { Field, Int, ObjectType } from 'type-graphql'
import { BaseSchema } from '../../lib/graphql/BaseSchema'
import { PwaWithOptin, PwaWithPublication } from './PwaDataSource'

@ObjectType()
export class PwaBlock {
  @Field() public type: string
  @Field({ nullable: true }) public condition?: string
  @Field(type => GraphQLJSONObject) public config: { [key: string]: string | number | string[] | number[] | boolean }
}

@ObjectType()
export class PwaSEOInfo {
  @Field({ nullable: true }) public type?: string
  @Field({ nullable: true }) public title?: string
  @Field({ nullable: true }) public description?: string
  @Field({ nullable: true }) public image?: string
  @Field({ nullable: true }) public imageAlt?: string
}

@ObjectType()
export class PwaSEODict {
  @Field(type => PwaSEOInfo) public home: PwaSEOInfo
  @Field(type => PwaSEOInfo) public articles: PwaSEOInfo
  @Field(type => PwaSEOInfo) public issues: PwaSEOInfo
  @Field(type => PwaSEOInfo) public account: PwaSEOInfo
}

@ObjectType()
export class PwaPushConfigGoogle {
  @Field({ nullable: true }) public serverPublicKey?: string
}

@ObjectType()
export class PwaPushDict {
  @Field(type => PwaPushConfigGoogle) public google: PwaPushConfigGoogle
}

@ObjectType()
export class PwaTrackingConfigFacebook {
  @Field({ nullable: true }) public pixelId?: string
  @Field({ nullable: true }) public appId?: string
}

@ObjectType()
export class PwaTrackingConfigGoogle {
  @Field({ nullable: true }) public measurementId?: string
}

@ObjectType()
export class PwaTrackingDict {
  @Field(type => PwaTrackingConfigFacebook) public facebook: PwaTrackingConfigFacebook
  @Field(type => PwaTrackingConfigGoogle) public google: PwaTrackingConfigGoogle
}

@ObjectType()
export class PwaPaymentConfigStripe {
  @Field() public publishableKey: string
}

@ObjectType()
export class PwaPaymentDict {
  @Field(type => PwaPaymentConfigStripe, { nullable: true }) public stripe?: PwaPaymentConfigStripe
}

@ObjectType()
export class PwaConfig {
  @Field() public live: boolean
  @Field(type => PwaSEODict) public seo: PwaSEODict
  @Field(type => PwaPushDict) public push: PwaPushDict
  @Field(type => PwaTrackingDict) public tracking: PwaTrackingDict
  @Field(type => PwaPaymentDict) public payment: PwaPaymentDict
}

@ObjectType()
export class PwaAction {
  @Field() public type: string
  @Field() public title: string
  @Field({ nullable: true }) public href?: string
  @Field({ nullable: true }) public target?: string
}

@ObjectType()
export class PwaBanner {
  @Field() public image: string
  @Field(type => PwaAction, { nullable: true }) public action?: PwaAction
}

@ObjectType()
export class PwaColumn {
  @Field(type => Int) public size: number
  @Field(type => [PwaBlock], { nullable: true }) public blocks: PwaBlock[]
}

@ObjectType()
export class PwaLayout {
  @Field(type => [PwaBanner]) public banners: PwaColumn[]
  @Field(type => [PwaColumn]) public columns: PwaColumn[]
}

@ObjectType()
export class PwaDesignImages {
  @Field() public icon: string
  @Field() public feature: string
}

@ObjectType()
export class PwaDesignStrings {
  @Field() public title: string
  @Field() public subtitle: string
}

@ObjectType()
export class PwaDesign {
  @Field(type => GraphQLJSONObject) public colors: { [key: string]: string }
  @Field(type => PwaDesignImages) public images: PwaDesignImages
  @Field(type => PwaDesignStrings) public strings: PwaDesignStrings
}

@ObjectType()
export class PwaCategory {
  @Field(type => Int) public id: number
  @Field({ nullable: true }) public slug?: string
  @Field() public color: string
  @Field() public title: string
  @Field(type => Int) public position: number
}

@ObjectType()
export class PwaOptinConfig {
  @Field({ nullable: true }) public image?: string
  @Field({ nullable: true }) public icon?: string
  @Field() public buttonColor: string
  @Field() public submitButtonLabel: string
  @Field() public color?: string
}

@ObjectType()
export class PwaOptin {
  @Field(type => Int) public id: number
  @Field() public title: string
  @Field() public description: string
  @Field(type => PwaOptinConfig) public config: PwaOptinConfig
}

@ObjectType()
export class Pwa extends BaseSchema<PwaWithPublication & PwaWithOptin> {
  @Field(type => Int) public id: number
  @Field() public domain: string
  @Field(type => Int) public publicationId: number
  @Field() public jobStatus: string
  @Field({ nullable: true }) public jobId?: string
  @Field() public createdAt: Date
  @Field() public updatedAt: Date
  @Field(type => PwaConfig) public config: PwaConfig
  @Field(type => PwaLayout) public layout: PwaLayout
  @Field(type => PwaDesign) public design: PwaDesign
  @Field(type => [PwaOptin], { nullable: true }) public PwaOptins: PwaOptin[]
  @Field(type => [PwaCategory]) public get categories(): PwaCategory[] { return this.model.Publication.Categories }
}
