import * as db from '@magloft/db-client'
import { DataSourceService } from '@tsed/graphql'
import { DataSource } from 'apollo-datasource'
import { Pwa } from './Pwa'

export type PublicationWithCategories = db.Publication & { Categories: db.Category[] }
export type PwaWithPublication = db.Pwa & { Publication?: PublicationWithCategories }
export type PwaWithOptin = db.Pwa & { PwaOptins?: db.PwaOptin[] }

@DataSourceService()
export class PwaDataSource extends DataSource {
  public async getById(id: number): Promise<Pwa> {
    // Retrieve PWA
    const include = [
      { model: db.Publication, as: 'Publication', required: true, include: [db.Publication.associations['Categories']] },
      { model: db.PwaOptin, as: 'PwaOptins', required: false }
    ]
    const pwa = await db.Pwa.findById<PwaWithPublication & PwaWithOptin>(id, { include })
    if (!pwa) { throw new Error('PWA App not found') }

    // Return PWA
    return new Pwa(pwa)
  }

  public async getByDomain(domain: string): Promise<Pwa> {
    // Retrieve PWA
    const include = [
      { model: db.Publication, as: 'Publication', required: true, include: [db.Publication.associations['Categories']] },
      { model: db.PwaOptin, as: 'PwaOptins', required: false }
    ]
    const pwa = await db.Pwa.findOne<PwaWithPublication & PwaWithOptin>({ where: { domain }, include })

    if (!pwa) { throw new Error('PWA App not found') }

    // Return PWA
    return new Pwa(pwa)
  }
}
