import * as db from '@magloft/db-client'
import { Field, Int, ObjectType } from 'type-graphql'
import { BaseSchema } from '../../lib/graphql/BaseSchema'
import { ReaderPurchase, ReaderPurchaseWithIncludes, ReaderSubscription, ReaderSubscriptionWithIncludes } from '../purchases/Purchase'

export type ReaderWithIncludes = db.Reader & {
  ReaderSubscriptions?: ReaderSubscriptionWithIncludes[]
  ReaderPurchases?: ReaderPurchaseWithIncludes[]
}

@ObjectType()
export class Reader extends BaseSchema<db.Reader> {
  @Field(type => Int) public id: number
  @Field() public email: string
  @Field() public confirmedAt: Date
  @Field({ nullable: true }) public firstname?: string
  @Field({ nullable: true }) public lastname?: string
  @Field() public source: string
  @Field(type => Int) public publicationId: number
  @Field() public newsletter: boolean
  @Field({ nullable: true }) public stripeId?: string
  @Field() public provider: string
  @Field() public uid: string
  @Field(type => [Int]) public classificationIds: number[]
  @Field({ nullable: true }) public token?: string
  @Field(type => [ReaderSubscription]) public subscriptions: ReaderSubscription[]
  @Field(type => [ReaderPurchase]) public purchases?: ReaderPurchase[]

  constructor(reader: ReaderWithIncludes, token?: string | null) {
    super(reader)
    this.firstname = reader.name
    if (token) { this.token = token }
    if (reader.newsletter == null) { this.newsletter = false }
    this.subscriptions = (reader.ReaderSubscriptions ?? []).map((item) => new ReaderSubscription(item))
    this.purchases = (reader.ReaderPurchases ?? []).map((item) => new ReaderPurchase(item))
  }
}
