import { IsEmail, MinLength } from 'class-validator'
import { ArgsType, Field, Int } from 'type-graphql'

@ArgsType()
export class GetByTokenArgs {
  @Field() token: string
}

@ArgsType()
export class SignInArgs {
  @Field(type => Int) publicationId: number
  @Field() @IsEmail() email: string
  @Field() @MinLength(6) password: string
}

@ArgsType()
export class SignUpArgs {
  @Field(type => Int) publicationId: number
  @Field() @IsEmail() email: string
  @Field({ nullable: true }) @MinLength(6) password?: string
  @Field({ nullable: true }) firstname?: string
  @Field({ nullable: true }) lastname?: string
  @Field({ nullable: true }) newsletter?: boolean
  @Field(type => Int, { nullable: true }) optinId?: number
  @Field(type => Int, { nullable: true }) articleId?: number
}

@ArgsType()
export class UpdateArgs {
  @Field() token: string
  @Field({ nullable: true }) email?: string
  @Field({ nullable: true }) password?: string
  @Field({ nullable: true }) firstname?: string
  @Field({ nullable: true }) lastname?: string
  @Field({ nullable: true }) newsletter?: boolean
  @Field(type => Int, { nullable: true }) optinId?: number
  @Field(type => Int, { nullable: true }) articleId?: number
}

@ArgsType()
export class RefreshArgs {
  @Field() token: string
}

@ArgsType()
export class SignOutArgs {
  @Field() token: string
}
