import { ResolverService } from '@tsed/graphql'
import { Args, Ctx, Mutation, Query } from 'type-graphql'
import { Reader } from './Reader'
import { ReaderDataSource } from './ReaderDataSource'
import { GetByTokenArgs, RefreshArgs, SignInArgs, SignOutArgs, SignUpArgs, UpdateArgs } from './ReaderInput'

interface DataSources {
  readerDataSource: ReaderDataSource
}

@ResolverService(Reader)
export class ReaderResolver {
  @Query(() => Reader)
  public async readerGetByToken(@Ctx('dataSources') { readerDataSource }: DataSources, @Args() args: GetByTokenArgs): Promise<Reader> {
    return readerDataSource.getByToken(args)
  }

  @Mutation(() => Reader)
  public async readerSignIn(@Ctx('dataSources') { readerDataSource }: DataSources, @Args() args: SignInArgs): Promise<Reader> {
    return readerDataSource.signIn(args)
  }

  @Mutation(() => Reader)
  public async readerSignUp(@Ctx('dataSources') { readerDataSource }: DataSources, @Args() args: SignUpArgs): Promise<Reader> {
    return readerDataSource.signUp(args)
  }

  @Mutation(() => Reader)
  public async readerUpdate(@Ctx('dataSources') { readerDataSource }: DataSources, @Args() args: UpdateArgs): Promise<Reader> {
    return readerDataSource.update(args)
  }

  @Mutation(() => Reader)
  public async readerRefresh(@Ctx('dataSources') { readerDataSource }: DataSources, @Args() args: RefreshArgs): Promise<Reader> {
    return readerDataSource.refresh(args)
  }

  @Mutation(() => Reader)
  public async readerSignOut(@Ctx('dataSources') { readerDataSource }: DataSources, @Args() args: SignOutArgs): Promise<Reader> {
    return readerDataSource.signOut(args)
  }
}
