import * as db from '@magloft/db-client'
import { Inject } from '@tsed/di'
import { ResolverService } from '@tsed/graphql'
import { Args, Authorized, Ctx, Mutation, Query } from 'type-graphql'
import { AuthService } from '../../services/AuthService'
import { User } from './User'
import { UserDataSource } from './UserDataSource'
import { GetByTokenArgs, SignInArgs, SignOutArgs } from './UserInput'

interface DataSources { userDataSource: UserDataSource }

export interface UserContext {
  dataSources: { userDataSource: UserDataSource }
  queryName: string
  queryFields: string[]
}

@ResolverService(User)
export class UserResolver {
  @Inject() auth: AuthService

  @Authorized('user')
  @Query(() => User)
  public async userGet(@Ctx('dataSources') { userDataSource }: DataSources, @Ctx('user') user: db.User): Promise<User> {
    const token = this.auth.encrypt(user)
    return userDataSource.getByToken({ token })
  }

  @Query(() => User)
  public async userGetByToken(@Ctx('dataSources') { userDataSource }: DataSources, @Args() args: GetByTokenArgs): Promise<User> {
    return userDataSource.getByToken(args)
  }

  @Mutation(() => User)
  public async userSignIn(@Ctx('dataSources') { userDataSource }: DataSources, @Args() args: SignInArgs): Promise<User> {
    return userDataSource.signIn(args)
  }

  @Mutation(() => User)
  public async userSignOut(@Ctx('dataSources') { userDataSource }: DataSources, @Args() args: SignOutArgs): Promise<User> {
    return userDataSource.signOut(args)
  }
}
