
export type EmailTemplateVariables = { [key: string]: string }
export type EmailTemplateLinks = { [key: string]: string }

const VARIABLE_REGEX = /<variable([^>]*)>\s*<\/variable>/g
const VAR_REGEX = /<var([^>]*)>\s*<\/var>/g
const LINK_REGEX = /\{\{\s*([a-zA-Z0-9_]+)\s*\}\}/g
const NAME_REGEX = /name=['"]([^'=]+)['"]/
const TYPE_REGEX = /type=['"]([^'=]+)['"]/
const VALUE_REGEX = /value=['"]([^'=]+)['"]/

export function parseEmailTemplate(html: string, variables: EmailTemplateVariables, links: EmailTemplateLinks) {
  html = html.replace(VARIABLE_REGEX, (_, value: string) => {
    if (!NAME_REGEX.test(value) || !TYPE_REGEX.test(value)) { return '' }
    const name = value.match(NAME_REGEX)[1].trim().toLowerCase()
    const type = value.match(TYPE_REGEX)[1].trim().toLowerCase()
    return (type === 'image') ? `<img height="auto" src="${variables[name]}" style="border:none;display:block;outline:none;text-decoration:none;width:100%;" width="100%">` : variables[name]
  })
  html = html.replace(VAR_REGEX, (_, value) => {
    if (!VALUE_REGEX.test(value)) { return '' }
    const name = value.match(VALUE_REGEX)[1].trim().toLowerCase()
    return variables[name] ?? ''
  })
  html = html.replace(LINK_REGEX, (_, value) => {
    return links[value] ?? ''
  })
  return html
}
