import * as db from '@magloft/db-client'
import { Pwa, Reader } from '@magloft/db-client'
import { Configuration } from '@tsed/di'

export type ProductType = 'issue' | 'subscription'
export type PaymentProviderIdentifier = 'stripe'
export enum PriceInterval { ONCE = 'once', DAY = 'day', WEEK = 'week', MONTH = 'month', YEAR = 'year' }
export enum UnlockKind { ISSUE = 'issue', SUBSCRIPTION = 'subscription', BUNDLE = 'bundle' }

export interface Price {
  id?: string
  currency: string
  amount: number
  interval: PriceInterval
  intervalCount?: number
}

export interface Unlock {
  kind: UnlockKind
  id: number
}

export interface Product {
  id: string
  provider: string
  name: string
  description: string
  images: string[]
  price: Price
  unlocks: Unlock[]
}

export interface CheckoutSession {
  id: string
  orderId: string
  provider: string
  product: Product
  readerId: number
  publicationId: number
  extra?: number
}

export interface BillingPortalSession {
  id: string
  provider: string
  readerId: number
  url: string
}

export interface CreateProductParams {
  id: string
  name: string
  description: string
  images: string[]
  price: Price
  unlocks: Unlock[]
}

export interface CreateCheckoutSessionParams {
  pwa: Pwa
  returnUrl: string
  product: Product
  reader: Reader
  extra?: number
}

export interface CreateBillingPortalParams {
  returnUrl: string
  reader: Reader
}

export interface PaymentProvider {
  configuration: Configuration
  pwa: Pwa
  syncProduct(params: CreateProductParams): Promise<Product>
  createBillingPortalSession(params: CreateBillingPortalParams): Promise<BillingPortalSession>
  createCheckoutSession(params: CreateCheckoutSessionParams): Promise<CheckoutSession>
  retrieveCheckoutSession(data: object): Promise<CheckoutSession>
}

export interface PaymentProviderStatic {
  new(configuration: Configuration, pwa: Pwa): PaymentProvider
}

export interface PaymentScope {
  publication: db.Publication,
  kind: string,
  id: number
}
