import { IMiddleware, Middleware, Next, Req, Res } from '@tsed/common'
import * as bodyParser from 'body-parser'

export type ReqWithBuffer = Req & { buffer?: Buffer }

@Middleware()
export class BodyParserMiddleware implements IMiddleware {
  private bodyParser = bodyParser.json({ limit: '50mb', verify: this.verify.bind(this) })

  public async use(@Req() req: Req, @Res() res: Res, @Next() next: Next) {
    return this.bodyParser(req, res, next)
  }

  private verify(req: ReqWithBuffer, res: Res, buffer: Buffer) {
    if (!req.header('stripe-signature')) { return }
    req.buffer = buffer
  }
}
