import { Pwa } from '@magloft/db-client'
import { Configuration, IMiddleware, Middleware, Next, Req, Res, Response } from '@tsed/common'
import * as cors from 'cors'
import { RequestHandler } from 'express'
import { AuthRequest } from './AuthMiddleware'

@Middleware()
export class CorsMiddleware implements IMiddleware {
  @Configuration() settings: Configuration
  private cors: RequestHandler = cors({ credentials: true, origin: this.origin.bind(this) })

  use(@Req() req: AuthRequest, @Res() res: Response, @Next() next: Next) {
    return this.cors(req, res, next)
  }

  private origin(requestOrigin: string | undefined, callback: (err: Error | null, allow?: boolean) => void) {
    const whitelist: string[] = this.settings.get('whitelist') ?? []
    if (!requestOrigin || whitelist.includes(requestOrigin)) { return callback(null, true) }
    if (requestOrigin.substr(0, 8) !== 'https://') { return callback(new Error('Not allowed by CORS'), false) }
    const domain = requestOrigin.substr(8)
    Pwa.findOne({ where: { domain } }).then(() => {
      callback(null, true)
    }).catch(() => {
      callback(new Error('Not allowed by CORS'))
    })
  }
}
