import { EndpointInfo, IMiddleware, Inject, Middleware, Next, Req, UseBefore } from '@tsed/common'
import { StoreSet, useDecorators } from '@tsed/core'
import { BadRequest } from '@tsed/exceptions'
import Stripe from 'stripe'
import { StripeService } from '../services/StripeService'
import { ReqWithBuffer } from './BodyParserMiddleware'

export type ReqWithStripeEvent = ReqWithBuffer & { event: Stripe.Event }

export interface VerifyStripeOptions {
  scope: 'pwa'
}

@Middleware()
export class VerifyStripeMiddleware implements IMiddleware {
  @Inject() stripe: StripeService

  public async use(@Req() req: ReqWithStripeEvent, @Next() next: Next, @EndpointInfo() endpoint: EndpointInfo) {
    const options = endpoint.get(VerifyStripeMiddleware) as VerifyStripeOptions
    const signature = req.header('stripe-signature')
    if (!signature) { throw new BadRequest('Missing Stripe Signature Header') }
    req.event = this.stripe.validateEvent(options.scope, signature, req.buffer)
    if (!req.event) { throw new BadRequest('Invalid Stripe Signature Header') }
    return next()
  }
}

export function VerifyStripe(scope: string) {
  return useDecorators(StoreSet(VerifyStripeMiddleware, { scope }), UseBefore(VerifyStripeMiddleware))
}
