import { close, connect, CustomFont } from '@magloft/db-client'
import * as kms from '@magloft/kms'
import { Configuration, DIConfiguration } from '@tsed/di'
import { PubSubService } from '../../services/PubSubService'
import * as fonts from './data/fonts.json'

connect(kms.config('credentials/db/cloud-server'))

async function main() {
  const config = await kms.config<Partial<Configuration>>('credentials/services/magloft-api')
  const configuration = new DIConfiguration(config) as Configuration
  const pubSub = new PubSubService(configuration)

  for (const font of fonts as { identifier: string, public: boolean }[]) {
    const [customFont, initialized] = await CustomFont.findOrBuild({ where: { identifier: font.identifier }, defaults: { variants: [] } })
    customFont.public = font.public
    await customFont.save()
    if (initialized) { await pubSub.trigger({ name: 'import-font', data: { url: `http://cdn.magloft.com/reader/fonts/${font.identifier}.css` } }) }
  }
  await close()
}

main().catch(console.error)
