import { Article, Op, Sequelize, WhereOptions } from '@magloft/db-client'
import { fromString } from 'html-to-text'
import { MigrationFn, runBatches } from '../../utils'

export const fixArticleInfo: MigrationFn = async (db: Sequelize, options: WhereOptions) => {
  const where = {
    info: { [Op.ne]: null },
    ...options
  }
  await runBatches<Article>(Article, where, async (articles) => {
    await db.transaction(async (transaction) => {
      for (const article of articles) {
        article.info = fromString(article.info, { wordwrap: false })
        await article.save({ transaction })
      }
    })
  }, 10)
}

