import { Article, Op, Sequelize, WhereOptions } from '@magloft/db-client'
import { extractText } from '@magloft/widgets/dist/server/ssr'
import { MigrationFn, runBatches } from '../../utils'

export const generateText: MigrationFn = async (db: Sequelize, options: WhereOptions) => {
  const where = {
    html: { [Op.ne]: null },
    text: { [Op.eq]: null },
    [Op.or]: {
      date: { [Op.ne]: null },
      issueId: { [Op.ne]: null }
    },
    ...options
  }
  await runBatches<Article>(Article, where, async (records) => {
    await db.transaction(async (transaction) => {
      for (const record of records) {
        record.text = extractText(record.html)
        await record.save({ transaction })
      }
    })
  }, 100)
}
