import { close, connect, CustomFont, Op, User } from '@magloft/db-client'
import * as kms from '@magloft/kms'
import { paramCase } from 'change-case'

connect(kms.config('credentials/db/cloud-server'))

async function main() {
  const allUsers = await User.findAll({ where: { properties: { [Op.ne]: null } } })
  const users = allUsers.filter((user) => user.properties.has('customFonts') && user.properties.get('customFonts').length > 0)

  // Update custom fonts
  for (const user of users) {
    const identifiers = user.properties.get('customFonts').map((title: string) => {
      const identifier = paramCase(title)
      if (identifier === 'bodoni-flf') { return 'bodoniflf' }
      if (identifier === 'breul-grotesk-a') { return 'breulgroteska' }
      if (identifier === 'breul-grotesk-b') { return 'breulgroteskb' }
      if (identifier === 'bebas-neue') { return 'bebasneue' }
      if (identifier === 'old-standard-tt') { return 'oldstandardtt' }
      if (identifier === 'six-caps') { return 'sixcaps' }
      if (identifier === 'spectra') { return 'spectral' }
      if (identifier === 'source-sans-pro-extra-light') { return 'source-sans-pro-extralight' }
      return identifier
    })
    user.properties.set('customFonts', identifiers)
    await user.save()
  }

  // Validate custom fonts are available
  const customFonts = await CustomFont.findAll()
  const identifiers = customFonts.map(({ identifier }) => identifier)
  const invalidFonts: string[] = []
  const invalidUsers: string[] = []
  for (const user of users) {
    for (const identifier of user.properties.get('customFonts') as string[]) {
      if (!identifiers.includes(identifier) && !invalidFonts.includes(identifier)) {
        invalidFonts.push(identifier)
        if (!invalidUsers.includes(user.email)) { invalidUsers.push(user.email) }
      }
    }
  }
  if (invalidFonts.length > 0) { throw new Error(`invalid fonts:\n${invalidFonts.join('\n')}`) }

  await close()
}

main().catch(console.error)
