import { Sequelize, WhereOptions } from '@magloft/db-client'
import { BaseModel } from '@magloft/db-client/lib/models/base/Model.base'

export type MigrationFn = (db: Sequelize, options: WhereOptions) => Promise<void>

export async function runBatches<T extends BaseModel>(Model: typeof BaseModel, where: WhereOptions, fn: (records: T[]) => Promise<void>, batchSize = 100) {
  const count = await Model.count({ where })
  const batchCount = Math.ceil(count / batchSize)
  const records = await Model.findAll({ where })
  const batches = groupBatches<T>(records as T[], batchSize)
  for (const batch of batches) {
    console.info(`~> running batch ${batches.indexOf(batch) + 1} of ${batchCount}`)
    await fn(batch)
  }
}

export function groupBatches<T>(records: T[], batchSize = 100): T[][] {
  return records.reduce<T[][]>((arr, record, index) => {
    const batchIndex = Math.floor(index / batchSize)
    if (arr.length < batchIndex + 1) { arr.push([]) }
    arr[batchIndex].push(record)
    return arr
  }, [])
}
