import * as db from '@magloft/db-client'
import * as kms from '@magloft/kms'
import * as mms from '@magloft/mms-db'
import '@tsed/ajv'
import { BeforeInit, BeforeRoutesInit, Configuration, GlobalAcceptMimesMiddleware, Inject, PlatformApplication } from '@tsed/common'
import '@tsed/platform-express'
import '@tsed/swagger'
import * as bodyParser from 'body-parser'
import * as cookieParser from 'cookie-parser'
import * as methodOverride from 'method-override'
import { config } from './config'
import { BodyParserMiddleware } from './middlewares/BodyParserMiddleware'
import { CorsMiddleware } from './middlewares/CorsMiddleware'
import { GraphQLAuthMiddleware } from './middlewares/GraphQLAuthMiddleware'

@Configuration(config)
export class Server implements BeforeInit, BeforeRoutesInit {
  @Inject() app: PlatformApplication
  @Inject() authMiddleware: GraphQLAuthMiddleware

  public async $beforeInit() {
    this.app.use(cookieParser())
    this.app.use('/graphql', this.authMiddleware.use.bind(this.authMiddleware))
  }

  public async $beforeRoutesInit() {
    db.connect(kms.config('credentials/db/cloud-server'), { logging: console.log })
    mms.connect(kms.config('credentials/db/mms'))
    this.app.use(GlobalAcceptMimesMiddleware)
    this.app.use(CorsMiddleware)
    this.app.use(methodOverride())
    this.app.use(BodyParserMiddleware)
    this.app.use(bodyParser.urlencoded({ extended: true }))
  }
}
