import { Attributes, PubSub } from '@google-cloud/pubsub'
import { Configuration, Injectable } from '@tsed/di'

export interface PubSubConfiguration {
  projectId: string
  apiEndpoint?: string
}

export type PubSubJob = { name: string, data: object, attributes?: Attributes } & (
  { name: 'import-font', data: { url: string } } |
  { name: 'pack-typeloft', data: { issueId: number } }
)

@Injectable()
export class PubSubService {
  private readonly config: PubSubConfiguration
  private readonly client: PubSub

  constructor(@Configuration() configuration: Configuration) {
    this.config = configuration.get<PubSubConfiguration>('pubSub')
    this.client = new PubSub(this.config)
  }

  trigger(job: PubSubJob): Promise<string> {
    const topic = this.client.topic(`drone-${job.name}`)
    return topic.publishJSON(job.data)
  }
}
