import { bootstrap } from '../../helpers'
import { RequestModule, RequestModuleContext } from '../../modules/RequestModule'

describe('ReaderSubscriptionController', () => {
  const ctx = bootstrap<RequestModuleContext>([RequestModule])

  describe('POST /portal/reader-subscriptions/:publicationId', () => {
    it('should create a reader subscription for a publication', async () => {
      await ctx.login('clienta@magloft.com', 'magloft')
      const startDate = new Date().getTime() / 1000
      const endDate = startDate + 60 * 60 * 24 * 360
      const response = await ctx.request.post('/portal/reader-subscriptions/3040').send({
        firstname: 'John',
        lastname: 'Doe',
        email: 'john.doe@magloft.com',
        password: 'test',
        skipConfirmation: true,
        externalId: 'TEST-USER-0001',
        orderId: 'TEST-ORDER-0001',
        startDate,
        endDate,
        active: true,
        customData: { foo: 'bar' },
        classificationIds: ['C1', 'C2']
      }).expect(200)
      expect(response.body).toBeInstanceOf(Object)
      expect(['created', 'updated']).toContain(response.body.status)
      expect(response.body.reader.email).toEqual('john.doe@magloft.com')
      expect(response.body.subscription.orderId).toEqual('TEST-ORDER-0001')
    })
  })
})
