import { Article, ArticleList } from '../../src/graphql/articles/Article'
import { ApolloModule, ApolloModuleContext, bootstrap, RequestModule, RequestModuleContext } from '../helpers'

describe('ArticleResolver', () => {
  const ctx = bootstrap<ApolloModuleContext & RequestModuleContext>([ApolloModule, RequestModule])

  it('loads an article by id', async () => {
    const query = `
      query articleGetById($id: Int!) {
        articleGetById(id: $id) {
          id
          title
          info
          publicationId
        }
      }`
    const article = await ctx.gqlQuery<Article>(query, 'articleGetById', { id: 461 })
    expect(article.publicationId).toEqual(102)
    expect(article.title).toEqual('Demo')
  })

  it('loads a list of articles', async () => {
    const query = `
      query articleGetList($publicationId: Int!) {
        articleGetList(publicationId: $publicationId) {
          offset
          limit
          records {
            id
            title
            info
            publicationId
          }
        }
      }`
    const articleList = await ctx.gqlQuery<ArticleList>(query, 'articleGetList', { publicationId: 102 })
    expect(typeof articleList.offset).toBe('number')
    expect(typeof articleList.limit).toBe('number')
    expect(articleList.offset).toEqual(0)
    expect(articleList.limit).toEqual(articleList.records.length)
  })
})
