import { Pwa } from '../../src/graphql/pwas/Pwa'
import { ApolloModule, ApolloModuleContext, bootstrap, RequestModule, RequestModuleContext } from '../helpers'

describe('PwaResolver', () => {
  const ctx = bootstrap<ApolloModuleContext & RequestModuleContext>([ApolloModule, RequestModule])

  it('loads a pwa by id', async () => {
    const query = `
      query pwaGetById($id: Int!) {
        pwaGetById(id: $id) {
          id
          domain
          publicationId
          jobStatus
          jobId
          createdAt
          updatedAt
          categories { id, color, title, position }
        }
      }`
    const pwa = await ctx.gqlQuery<Pwa>(query, 'pwaGetById', { id: 1 })
    expect(pwa.publicationId).toEqual(102)
    expect(pwa.domain).toEqual('pwa.magloft.dev')
  })

  it('loads a pwa by domain', async () => {
    const query = `
      query pwaGetByDomain($domain: String!) {
        pwaGetByDomain(domain: $domain) {
          id
          domain
          publicationId
          jobStatus
          jobId
          createdAt
          updatedAt
          categories { id, color, title, position }
        }
      }`
    const pwa = await ctx.gqlQuery<Pwa>(query, 'pwaGetByDomain', { domain: 'pwa.magloft.dev' })
    expect(pwa.publicationId).toEqual(102)
    expect(pwa.domain).toEqual('pwa.magloft.dev')
  })
})
