import * as db from '@magloft/db-client'
import * as kms from '@magloft/kms'
import * as mms from '@magloft/mms-db'
import { Configuration, PlatformTest, Res } from '@tsed/common'
import { config } from '../src/config'
import { Server } from '../src/Server'

export * from './modules/ApolloModule'
export * from './modules/RequestModule'

export interface BootstrapModule<C> {
  config: (options: Partial<Configuration>) => Promise<unknown>,
  context: (test: typeof PlatformTest) => Promise<Partial<C>>
}

export function createInjectModule<C extends {}, T>(key: string, target: T): BootstrapModule<C> {
  const InjectModule: BootstrapModule<C> = {
    config: async () => { },
    context: async (test) => {
      return { [key]: test.get<T>(target) } as C
    }
  }
  return InjectModule
}

export function bootstrap<T>(modules: BootstrapModule<T>[]) {
  const context: Partial<T> = {}
  beforeAll(async () => {
    const serviceConfig = await kms.config<Partial<Configuration>>('credentials/services/magloft-api')
    const options: Partial<Configuration> = { ...config, ...serviceConfig }
    for (const currentModule of modules) {
      if (currentModule.config) {
        await currentModule.config(options)
      }
    }
    await PlatformTest.bootstrap(Server, options)()
    for (const currentModule of modules) {
      const moduleContext = await currentModule.context(PlatformTest)
      Object.assign(context, moduleContext)
    }
  })
  afterAll(PlatformTest.reset)
  afterAll(db.close)
  afterAll(mms.close)
  return context as T
}

export function mockResponse() {
  const res = {} as Res
  res.status = jest.fn().mockReturnValue(res)
  res.json = jest.fn().mockReturnValue(res)
  res.cookie = jest.fn().mockReturnValue(res)
  res.clearCookie = jest.fn().mockReturnValue(res)
  return res
}
