import { User } from '@magloft/db-client'
import { GraphQLService } from '@tsed/graphql'
import { ApolloServerBase } from 'apollo-server-core'
import { ApolloServerTestClient, createTestClient } from 'apollo-server-testing'
import { DocumentNode } from 'graphql'
import { AdminArticleDataSource } from '../../src/graphql/admin/articles/AdminArticleDataSource'
import { ArticleDataSource } from '../../src/graphql/articles/ArticleDataSource'
import { CategoryDataSource } from '../../src/graphql/categories/CategoryDataSource'
import { IssueDataSource } from '../../src/graphql/issues/IssueDataSource'
import { PurchaseDataSource } from '../../src/graphql/purchases/PurchaseDataSource'
import { PwaDataSource } from '../../src/graphql/pwas/PwaDataSource'
import { BootstrapModule } from '../helpers'

export interface ApolloModuleContext {
  service: GraphQLService,
  server: ApolloServerBase,
  client: ApolloServerTestClient
  gqlQuery: <T>(query: string | DocumentNode, operationName: string, variables: { [name: string]: unknown }) => Promise<T>
}

const gqlContext: { user?: User, role?: string } = {}

export const ApolloModule: BootstrapModule<ApolloModuleContext> = {
  config: async (options) => {
    options.graphql.magloft.serverConfig.context = () => gqlContext
    options.graphql.magloft.serverConfig.dataSources = () => ({
      adminArticleDataSource: new AdminArticleDataSource(),
      articleDataSource: new ArticleDataSource(),
      categoryDataSource: new CategoryDataSource(),
      issueDataSource: new IssueDataSource(),
      purchaseDataSource: new PurchaseDataSource(),
      pwaDataSource: new PwaDataSource()
    })
  },
  context: async (test) => {
    gqlContext.user = await User.findById(1)
    gqlContext.role = gqlContext.user.role
    const service = test.get<GraphQLService>(GraphQLService)
    const server = service.get('magloft')
    const client = createTestClient(server)
    const gqlQuery = async <T>(query: string | DocumentNode, operationName: string, variables: { [name: string]: unknown }) => {
      const { data, errors } = await client.query({ query, variables, operationName })
      if (errors) { throw new Error(errors.map((error) => error.message).join('\n')) }
      return data[operationName] as T
    }
    return { service, server, client, gqlQuery }
  }
}
